@@ -2,8 +2,10 @@ pub extern crate termion;
 
 mod utils;
 pub mod pixel;
+pub mod screen;
 
 use pixel::Pixel;
+use screen::Screen;
 use termion::color;
 use termion::event::{Event, Key};
 use std::io::{stdout, Stdout};
@@ -11,7 +13,7 @@ use termion::raw::IntoRawMode;
 use std::io::Write;
 use termion::input::{TermRead, MouseTerminal};
 
-/// # Console Engine Framework
+/// Console Engine Framework
 pub struct ConsoleEngine {
     input: termion::input::Events<termion::AsyncReader>,
     output: MouseTerminal<termion::raw::RawTerminal<Stdout>>,
@@ -20,8 +22,8 @@ pub struct ConsoleEngine {
     pub frame_count: usize,
     width: u32,
     height: u32,
-    screen: Vec<Pixel>,
-    screen_last_frame: Vec<Pixel>,
+    screen: Screen,
+    screen_last_frame: Screen,
     instant: std::time::Instant,
     keys_pressed: Vec<Event>,
     keys_held: Vec<Event>,
@@ -70,12 +72,12 @@ impl ConsoleEngine {
             frame_count: 0,
             width: width,
             height: height,
-            screen: vec![pixel::pxl(' '); (width*height) as usize],
-            screen_last_frame: vec![],
+            screen: Screen::new(width, height),
+            screen_last_frame: Screen::from_vec(vec![], width, height),
             instant: std::time::Instant::now(),
-            keys_pressed: vec!(),
-            keys_held: vec!(),
-            keys_released: vec!(),
+            keys_pressed: vec![],
+            keys_held: vec![],
+            keys_released: vec![],
         };
         my.begin();
         return my;
@@ -114,20 +116,33 @@ impl ConsoleEngine {
     }
 
     /// Get the screen width
+    #[deprecated(since="0.6.0", note="please use `get_width` instead. This function will be removed in version 1.0")]
     pub fn scr_w(&self) -> u32
     {
-        self.width
+        self.screen.get_width()
     }
+    /// Get the screen width
+    pub fn get_width(&self) -> u32
+    {
+        self.screen.get_width()
+    }
+
     /// Get the screen height
+    #[deprecated(since="0.6.0", note="please use `get_height` instead. This function will be removed in version 1.0")]
     pub fn scr_h(&self) -> u32
     {
-        self.height
+        self.screen.get_height()   
+    }
+    /// Get the screen height
+    pub fn get_height(&self) -> u32
+    {
+        self.screen.get_height()   
     }
 
     /// Reset the screen to a blank state
     pub fn clear_screen(&mut self) 
     {
-        self.screen = vec![pixel::pxl(' '); (self.width*self.height) as usize];
+        self.screen.clear()
     }
 
     /// prints a string at the specified coordinates.  
@@ -140,29 +155,7 @@ impl ConsoleEngine {
     /// ```
     pub fn print(&mut self, x: i32, y: i32, string: String)
     {
-        if y >= 0 && x < self.width as i32 && y < self.height as i32 {
-            // get screen index, initializes a counter 
-            // and get chars of the provided String
-            let pos = self.coord_to_index(std::cmp::max(0,x), y);
-            let mut delta_x = 0usize;
-            if x < 0 {
-                delta_x = x.abs() as usize;
-            }
-            let mut count = delta_x;
-            let char_vec: Vec<char> = string.chars().collect();
-            let origin_row = pos/self.scr_w() as usize;
-            // place each characters one by one. Stops before overflowing
-            for i in pos..std::cmp::min(pos+char_vec.len()-delta_x, self.screen.capacity()) {
-                // if the row changes, break. 
-                // removing this statement will cause a wrapping of the text
-                if origin_row != i/self.scr_w() as usize {
-                    break;
-                }
-                // print the character on screen
-                self.screen[i] = pixel::pxl(char_vec[count]);
-                count += 1;
-            }
-        }
+        self.screen.print(x,y,string)
     }
 
     /// prints a string at the specified coordinates with the specified foreground and background color  
@@ -175,29 +168,30 @@ impl ConsoleEngine {
     /// ```
     pub fn print_fbg<C1: color::Color + Clone, C2: color::Color + Clone>(&mut self, x: i32, y: i32, string: String, fg: C1, bg: C2)
     {
-        if y >= 0 && x < self.width as i32 && y < self.height as i32 {
-            // get screen index, initializes a counter 
-            // and get chars of the provided String
-            let pos = self.coord_to_index(std::cmp::max(0,x), y);
-            let mut delta_x = 0usize;
-            if x < 0 {
-                delta_x = x.abs() as usize;
-            }
-            let mut count = delta_x;
-            let char_vec: Vec<char> = string.chars().collect();
-            let origin_row = pos/self.scr_w() as usize;
-            // place each characters one by one. Stops before overflowing
-            for i in pos..std::cmp::min(pos+char_vec.len()-delta_x, self.screen.capacity()) {
-                // if the row changes, break. 
-                // removing this statement will cause a wrapping of the text
-                if origin_row != i/self.scr_w() as usize {
-                    break;
-                }
-                // print the character on screen
-                self.screen[i] = pixel::pxl_fbg(char_vec[count], fg.clone(), bg.clone());
-                count += 1;
-            }
-        }
+        self.screen.print_fbg(x, y, string, fg, bg)
+    }
+
+    /// Prints another screen on specified coordinates.
+    /// Useful when you want to manage several "subscreen"
+    /// 
+    /// *see example* `screen-embed`
+    /// 
+    /// usage :
+    /// ```
+    /// use console_engine::pixel;
+    /// use console_engine::screen::Screen;
+    /// 
+    /// // create a new Screen struct and draw a square inside it
+    /// let mut my_square = Screen::new(8,8);
+    /// my_square.rect(0,0,7,7,pixel::pxl('#'));
+    /// my_square.print(1,1,String::from("square"));
+    /// 
+    /// // prints the square in the engine's screen at a specific location
+    /// engine.print_screen(5,2, &my_square);
+    /// ```
+    pub fn print_screen(&mut self, x : i32, y: i32, source: &Screen)
+    {
+        self.screen.print_screen(x, y, source)
     }
 
     /// draws a line of the provided character between two sets of coordinates  
@@ -213,92 +207,7 @@ impl ConsoleEngine {
     /// ```
     pub fn line(&mut self, start_x: i32, start_y: i32, end_x: i32, end_y: i32, character: Pixel)
     {
-        let delta_x = end_x - start_x;
-        let delta_y = end_y - start_y;
-        // optimized algorithms for pure horizontal or vertical lines
-        if delta_y == 0 {
-            let mut start = start_x;
-            let mut end = end_x+1;
-            if end_x < start_x {
-                end = start_x+1;
-                start = end_x;
-            };
-            // horizontal line
-            for i in start..end {
-                self.set_pxl_ref(i, start_y, &character);
-            }
-            return;
-        }
-        if delta_x == 0 {
-            let mut start = start_y;
-            let mut end = end_y+1;
-            if end_y < start_y {
-                end = start_y+1;
-                start = end_y;
-            };
-            // horizontal line
-            for j in start..end {
-                self.set_pxl_ref(start_x, j, &character);
-            }
-            return;
-        }
-
-        // Bresenham's line algorithm
-        let line_low = |engine: &mut ConsoleEngine, x0: i32,y0: i32, x1: i32,y1: i32| {
-            let dx: i32 = x1 - x0;
-            let mut dy: i32 = y1 - y0;
-            let mut yi = 1;
-            if dy < 0 {
-                yi = -1;
-                dy = -dy;
-            }
-            let mut d = 2*dy - dx;
-            let mut y = y0;
-
-            for x in x0..x1+1 {
-                engine.set_pxl_ref(x, y, &character);
-                if d > 0 {
-                    y = y + yi;
-                    d = d - 2*dx;
-                }
-                d = d + 2*dy;
-            } 
-        };
-
-        let line_high = |engine: &mut ConsoleEngine, x0: i32,y0: i32, x1: i32,y1: i32| {
-            let mut dx = x1 - x0;
-            let dy = y1 - y0;
-            let mut xi = 1;
-            if dx < 0 {
-                xi = -1;
-                dx = -dx;
-            }
-            let mut d = 2*dx - dy;
-            let mut x = x0;
-        
-            for y in y0..y1+1 {
-                engine.set_pxl_ref(x, y, &character);
-                if d > 0 {
-                    x = x + xi;
-                    d = d - 2*dy;
-                }
-                d = d + 2*dx;
-            }   
-        };
-
-        if (end_y - start_y).abs() < (end_x - start_x).abs() {
-            if start_x > end_x {
-                line_low(self, end_x, end_y, start_x, start_y);
-            } else {
-                line_low(self, start_x, start_y, end_x, end_y);
-            }
-        } else {
-            if start_y > end_y {
-                line_high(self, end_x, end_y, start_x, start_y);
-            } else {
-                line_high(self, start_x, start_y, end_x, end_y);
-            }
-        }
+        self.screen.line(start_x, start_y, end_x, end_y, character)
     }
 
     /// Draws a rectangle of the provided character between two sets of coordinates  
@@ -311,10 +220,7 @@ impl ConsoleEngine {
     /// ```
     pub fn rect(&mut self, start_x: i32, start_y: i32, end_x: i32, end_y: i32, character: Pixel)
     {
-        self.line(start_x, start_y, end_x, start_y, character.clone()); // top
-        self.line(end_x, start_y, end_x, end_y, character.clone());     // right
-        self.line(end_x, end_y, start_x, end_y, character.clone());     // bottom
-        self.line(start_x, end_y, start_x, start_y, character.clone()); // left
+        self.screen.rect(start_x, start_y, end_x, end_y, character)
     }
 
     /// Fill a rectangle of the provided character between two sets of coordinates  
@@ -327,11 +233,7 @@ impl ConsoleEngine {
     /// ```
     pub fn fill_rect(&mut self, start_x: i32, start_y: i32, end_x: i32, end_y: i32, character: Pixel)
     {
-        let y0 = if start_y < end_y { start_y } else { end_y };
-        let y1 = if start_y < end_y { end_y+1 } else { start_y+1 };
-        for y in y0..y1 {
-            self.line(start_x, y, end_x, y, character.clone());
-        }
+        self.screen.fill_rect(start_x, start_y, end_x, end_y, character)
     }
 
     /// Draws a circle of the provided character at an x and y position with a radius
@@ -345,32 +247,7 @@ impl ConsoleEngine {
     /// ```
     pub fn circle(&mut self, x: i32, y: i32, radius: u32, character: Pixel)
     {
-        let mut relative_pos_x = 0 as i32;
-		let mut relative_pos_y = radius as i32;
-		let mut distance: i32 = 3 - 2 * radius as i32;
-		if radius == 0 {
-            return;
-        }
-
-		while relative_pos_y >= relative_pos_x
-		{
-			self.set_pxl_ref(x + relative_pos_x, y - relative_pos_y, &character);
-			self.set_pxl_ref(x + relative_pos_y, y - relative_pos_x, &character);
-			self.set_pxl_ref(x + relative_pos_y, y + relative_pos_x, &character);
-			self.set_pxl_ref(x + relative_pos_x, y + relative_pos_y, &character);
-			self.set_pxl_ref(x - relative_pos_x, y + relative_pos_y, &character);
-			self.set_pxl_ref(x - relative_pos_y, y + relative_pos_x, &character);
-			self.set_pxl_ref(x - relative_pos_y, y - relative_pos_x, &character);
-			self.set_pxl_ref(x - relative_pos_x, y - relative_pos_y, &character);
-			if distance < 0 {
-                distance += 4 * relative_pos_x as i32 + 6;
-                relative_pos_x += 1;
-            } else {
-                distance += 4 * (relative_pos_x as i32 - relative_pos_y as i32) + 10;
-                relative_pos_x += 1;
-                relative_pos_y -= 1;
-            } 
-		}
+        self.screen.circle(x, y, radius, character)
     }
 
     /// Fill a circle of the provided character at an x and y position with a radius
@@ -380,42 +257,11 @@ impl ConsoleEngine {
     /// ```
     /// use console_engine::pixel;
     /// // ...
-    /// engine.circle(10, 10, 4, pixel::pxl('#'));
+    /// engine.fill_circle(10, 10, 4, pixel::pxl('#'));
     /// ```
     pub fn fill_circle(&mut self, x: i32, y: i32, radius: u32, character: Pixel)
     {
-        // Taken from wikipedia
-		let mut relative_pos_x = 0 as i32;
-		let mut relative_pos_y = radius as i32;
-		let mut distance: i32 = 3 - 2 * radius as i32;
-		if radius == 0 {
-            return;
-        }
-
-        // create a lambda function that draw fast horizontal lines
-		let mut drawline = |start_x: i32, end_x: i32, y: i32|
-		{
-			for i in start_x..end_x+1 {
-				self.set_pxl_ref(i, y, &character);
-            }
-		};
-
-		while relative_pos_y >= relative_pos_x
-		{
-			// Modified to draw scan-lines instead of edges
-			drawline(x - relative_pos_x, x + relative_pos_x, y - relative_pos_y);
-			drawline(x - relative_pos_y, x + relative_pos_y, y - relative_pos_x);
-			drawline(x - relative_pos_x, x + relative_pos_x, y + relative_pos_y);
-			drawline(x - relative_pos_y, x + relative_pos_y, y + relative_pos_x);
-			if distance < 0 {
-                distance += 4 * relative_pos_x + 6;
-                relative_pos_x += 1;
-            } else {
-                distance += 4 * (relative_pos_x - relative_pos_y) + 10;
-                relative_pos_x += 1;
-                relative_pos_y -= 1;
-            } 
-		}
+        self.screen.fill_circle(x, y, radius, character)
     }
 
     /// Draws a triangle of the provided character using three sets of coordinates
@@ -428,9 +274,7 @@ impl ConsoleEngine {
     /// ```
     pub fn triangle(&mut self, x1: i32, y1: i32, x2: i32, y2: i32, x3: i32, y3: i32, character: Pixel)
     {
-        self.line(x1, y1, x2, y2, character.clone());
-        self.line(x2, y2, x3, y3, character.clone());
-        self.line(x3, y3, x1, y1, character.clone());
+        self.screen.triangle(x1, y1, x2, y2, x3, y3, character)
     }
 
     /// Fill a triangle of the provided character using three sets of coordinates
@@ -444,90 +288,7 @@ impl ConsoleEngine {
     /// ```
     pub fn fill_triangle(&mut self, x1: i32, y1: i32, x2: i32, y2: i32, x3: i32, y3: i32, character: Pixel)
     {
-        self.triangle(x1 as i32, y1 as i32, x2 as i32, y2 as i32, x3 as i32, y3 as i32, character.clone());
-        // we use tuples for this for now
-        let v0 = (x1 as i32, y1 as i32);
-        let mut v1 = (x2 as i32, y2 as i32);
-        let mut v2 = (x3 as i32, y3 as i32);
-
-        // algorithm only fills counter clockwise triangles, so swap as needed
-        // For a triangle A B C, you can find the winding by computing the cross product (B - A) x (C - A). For 2d tri's, with z=0, it will only have a z component.
-        // To give all the same winding, swap vertices C and B if this z component is negative.
-        let cross = (v1.1 - v0.1) * (v2.0 - v1.0) - (v1.0 - v0.0) * (v2.1 - v1.1); 
-        if cross > 0 { std::mem::swap(&mut v1, &mut v2) }
-        
-        // Compute triangle bounding box and clip to screen bounds
-        let min_x = std::cmp::max(std::cmp::min(std::cmp::min(v0.0, v1.0), v2.0), 0);
-        let max_x = std::cmp::min(std::cmp::max(std::cmp::max(v0.0, v1.0), v2.0), self.scr_w() as i32 - 1);
-        let min_y = std::cmp::max(std::cmp::min(std::cmp::min(v0.1, v1.1), v2.1), 0);
-        let max_y = std::cmp::min(std::cmp::max(std::cmp::max(v0.1, v1.1), v2.1), self.scr_h() as i32 - 1);
-
-        // Triangle setup
-        let a01 = v0.1 - v1.1;
-        let b01 = v1.0 - v0.0;
-        let a12 = v1.1 - v2.1;
-        let b12 = v2.0 - v1.0;
-        let a20 = v2.1 - v0.1;
-        let b20 = v0.0 - v2.0;
-
-        // Determine edges
-        let is_top_left = |v0: (i32, i32), v1: (i32, i32)| -> bool {
-            v0.1 > v1.1 
-        };
-
-        // We follow fill rules and add a bias
-        let bias0 = if is_top_left(v1, v2) { 0 } else { -1 };
-        let bias1 = if is_top_left(v2, v0) { 0 } else { -1 };
-        let bias2 = if is_top_left(v0, v1) { 0 } else { -1 };
-
-        // Determine barycentric coordinates
-        let orient2d = |a: (i32,i32), b: (i32,i32), c: (i32,i32)| -> i32 {
-            (b.0-a.0)*(c.1-a.1) - (b.1-a.1)*(c.0-a.0)
-        };
-
-        let mut p = (min_x, min_y);
-        let mut w0_row = orient2d(v1, v2, p) + bias0;
-        let mut w1_row = orient2d(v2, v0, p) + bias1;
-        let mut w2_row = orient2d(v0, v1, p) + bias2;
-
-        // Rasterize
-        for y in min_y..max_y {
-            p.1 = y;
-            // Barycentric coordinates at start of row
-            let mut w0 = w0_row;
-            let mut w1 = w1_row;
-            let mut w2 = w2_row;
-
-                for x in min_x..max_x {
-                    p.0 = x;
-                    // If p is on or inside all edges, render pixel.
-                    if (w0 | w1 | w2) >= 0 {
-                        self.set_pxl_ref(p.0, p.1, &character);
-                    }
-
-                    // One step to the right
-                    w0 += a12;
-                    w1 += a20;
-                    w2 += a01;
-                }
-            // One row step
-            w0_row += b12;
-            w1_row += b20;
-            w2_row += b01;
-        }
-    }
-
-
-    /// Referenced version of set_pxl  
-    /// see set_pxl for more information on this usage
-    /// 
-    /// The only differences between the two is that this version takes the Pixel as a reference
-    fn set_pxl_ref(&mut self, x: i32, y: i32, character: &Pixel)
-    {
-        if x >= 0 && y >= 0 && x < self.width as i32 && y < self.height as i32 {
-            let index = self.coord_to_index(x, y);
-            self.screen[index] = character.clone();
-        }
+        self.screen.fill_triangle(x1, y1, x2, y2, x3, y3, character)
     }
 
     /// sets the provided character in the specified coordinates
@@ -541,10 +302,7 @@ impl ConsoleEngine {
     /// ```
     pub fn set_pxl(&mut self, x: i32, y: i32, character: Pixel)
     {
-        if x >= 0 && y >= 0 && x < self.width as i32 && y < self.height as i32 {
-            let index = self.coord_to_index(x, y);
-            self.screen[index] = character;
-        }
+        self.screen.set_pxl(x, y, character)
     }
 
     /// Get the character stored at provided coordinates
@@ -557,10 +315,42 @@ impl ConsoleEngine {
     /// ```
     pub fn get_pxl(&self, x: i32, y: i32) -> Result<Pixel, String> 
     {
-        if x >= 0 && y >= 0 && x < self.width as i32 && y < self.height as i32 {
-            return Ok(self.screen[self.coord_to_index(x, y)].clone());
-        }
-        Err(format!("Attempted to get_pxl out of bounds (coords: [{}, {}], bounds: [{}, {}]", x,y,self.width-1,self.height-1))
+        self.screen.get_pxl(x,y)
+    }
+
+    /// Resizes the screen to match the given width and height
+    /// truncates the bottom and right side of the screen
+    /// 
+    /// usage:
+    /// ```
+    /// engine.resize()
+    /// ```
+    pub fn resize(&mut self, new_width: u32, new_height: u32)
+    {
+        self.screen.resize(new_width, new_height);
+        self.width = new_width;
+        self.height = new_height;
+        self.screen_last_frame = Screen::from_vec(vec![], self.width, self.height);
+    }
+
+    /// Changes the screen instance used by the engine and updates internal informations
+    /// 
+    /// Useful if you want to manage multiple screens independently.
+    pub fn set_screen(&mut self, screen: Screen)
+    {
+        self.width = screen.get_width();
+        self.height = screen.get_height();
+        self.screen = screen;
+        self.screen_last_frame = Screen::from_vec(vec![], self.width, self.height);
+    }
+
+    /// Returns a clone of the current screen
+    /// 
+    /// You can keep it into a variable to restore the screen later, via `set_screen`.
+    /// You can then use the to_string method to write the screen in a file for example
+    pub fn get_screen(&self) -> Screen
+    {
+        self.screen.clone()
     }
     
     /// Draw the screen in the terminal  
@@ -582,14 +372,16 @@ impl ConsoleEngine {
         output_screen.push_str(&format!("{}", termion::cursor::Goto(1,1)));
         let mut current_colors = String::from("");
         let mut moving = false;
+        let screen = self.screen.open_screen();
+        let screen_last_frame = self.screen_last_frame.open_screen().screen;
         // iterates through the screen memory and prints it on the output buffer
         for y in 0..self.height {
             for x in 0..self.width {
-                let index = self.coord_to_index(x as i32, y as i32);
-                let pixel = &self.screen[index];
+                let index = screen.coord_to_index(x as i32, y as i32);
+                let pixel = &screen[index];
                 // we check if the screen has been modified at this coordinate
                 // if so, we write like normally, else we set a 'moving' flag
-                if self.screen_last_frame.is_empty() || *pixel != self.screen_last_frame[index] {
+                if screen_last_frame.is_empty() || *pixel != screen_last_frame[index] {
                     if moving {
                         // if the moving flag is set, we need to write a goto instruction first
                         // this optimization minimize useless write on the screen
@@ -604,7 +396,7 @@ impl ConsoleEngine {
                     // time consuming
                     if current_colors != pixel.colors {
                         current_colors = pixel.colors.clone();
-                        output_screen.push_str(&format!("{}", pixel));
+                        output_screen.push_str(pixel.to_string().as_str());
                     } else {
                         output_screen.push(pixel.chr);
                     }
@@ -681,22 +473,8 @@ impl ConsoleEngine {
             let size = termion::terminal_size().unwrap();
             let new_width = size.0 as u32;
             let new_height = size.1 as u32;
-            // create new screens Vec
-            let mut new_screen = vec![pixel::pxl(' '); (size.0*size.1) as usize];
-            let mut new_screen_last_frame = vec![pixel::pxl(' '); (size.0*size.1) as usize];
-            // transfer old screens into new screens
-            for j in 0..std::cmp::min(self.height, new_height) {
-                for i in 0..std::cmp::min(self.width, new_width) {
-                    if (i as u32) < self.width && (j as u32) < self.height {
-                        new_screen[((j*new_width)+i) as usize] = self.screen[((j*self.width)+i) as usize].clone();
-                        new_screen_last_frame[((j*new_width)+i) as usize] = self.screen_last_frame[((j*self.width)+i) as usize].clone();
-                    }
-                }
-            }
-            self.screen = new_screen;
-            self.screen_last_frame = vec![];
-            self.width = new_width;
-            self.height = new_height;
+
+            self.resize(new_width, new_height);
         }
     }
 
@@ -817,10 +595,10 @@ impl ConsoleEngine {
     /// usage :
     /// ```
     /// // prints a 'R' where the mouse has been released
-    /// let mouse_pos = engine.get_mouse_held();
+    /// let mouse_pos = engine.get_mouse_released();
     /// if mouse_pos.is_some() {
     ///     let mouse_pos = mouse_pos.unwrap();
-    ///     engine.set_pxl(mouse_pos.0 as i32, mouse_pos.1 as i32, pixel::pxl('H'));
+    ///     engine.set_pxl(mouse_pos.0 as i32, mouse_pos.1 as i32, pixel::pxl('R'));
     /// }
     /// ```
     pub fn get_mouse_released(&self) -> Option<(u32,u32)> 
@@ -840,15 +618,6 @@ impl ConsoleEngine {
         }
         return None;
     }
-
-    /// Converts x and y coordinates to screen index
-    /// 
-    /// example : on a 10x10 screen
-    /// `coord_to_index(2,1)` will return index 12
-    fn coord_to_index(&self, x: i32, y: i32) -> usize
-    {
-        return ((y*self.width as i32) + x) as usize;
-    }
 }
 
 impl Drop for ConsoleEngine {
